/* AUTO-GENERATED FILE.  DO NOT MODIFY.
 *
 * This class was automatically generated by the
 * aapt tool from the resource data it found.  It
 * should not be modified by hand.
 */

package com.max2idea.android.fwknop;

public final class R {
    public static final class attr {
    }
    public static final class drawable {
        public static final int icon=0x7f020000;
        public static final int lock_128x128=0x7f020001;
        public static final int lock_32x32=0x7f020002;
        public static final int lock_64x64=0x7f020003;
    }
    public static final class id {
        public static final int accessPort=0x7f050008;
        public static final int accessPortStr=0x7f050007;
        public static final int accessPortl=0x7f050006;
        public static final int accessProto=0x7f050005;
        public static final int accessProtoStr=0x7f050004;
        public static final int accessProtol=0x7f050003;
        public static final int allowip=0x7f050002;
        public static final int allowipl=0x7f050000;
        public static final int allowipstr=0x7f050001;
        public static final int destIP=0x7f05000b;
        public static final int destIPStr=0x7f05000a;
        public static final int destIPl=0x7f050009;
        public static final int fwTimeout=0x7f05000e;
        public static final int fwTimeoutStr=0x7f05000d;
        public static final int fwTimeoutl=0x7f05000c;
        public static final int output=0x7f050016;
        public static final int passwd=0x7f050011;
        public static final int passwdStr=0x7f050010;
        public static final int passwdl=0x7f05000f;
        public static final int startApp=0x7f050013;
        public static final int startAppCheck=0x7f050014;
        public static final int startAppl=0x7f050012;
        public static final int startl=0x7f050015;
        public static final int unlock=0x7f050017;
    }
    public static final class layout {
        public static final int main=0x7f030000;
    }
    public static final class string {
        public static final int app_name=0x7f040000;
        public static final int app_short_name=0x7f040001;
    }
}
